unit prepravka_u;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils;

type
  TPrepravka = class
    private
      x, y, z: real;
      pocet: integer;
      zamek: TRTLCriticalSection;
    public
      constructor Create;
      Destructor Destroy; override;
      procedure prijmi(gama1, gama2: real);
      function data: string;
      procedure kontrola;
      property Text: string read data;
  end;

var prepravka: TPrepravka;

implementation

const
  epsilon = 1e-11;

constructor TPrepravka.Create;
begin
  x := 1; y := 0; z := 0;
  pocet := 0;
  InitCriticalSection(zamek);
end;

destructor TPrepravka.Destroy;
begin
  DoneCriticalSection(zamek);
end;

procedure TPrepravka.prijmi(gama1, gama2: real);
  var
    fi, costheta, sintheta: real;
begin
  costheta:=2*gama1 - 1;
  sintheta:=sqrt(1 - sqr(costheta));
  fi := 2*gama2 * pi;
  EnterCriticalSection(zamek);
  try
    x:=sintheta*cos(fi);
    y:=sintheta*sin(fi);
    z:=costheta;
    inc(pocet);
  finally
    LeaveCriticalSection(zamek);
  end;
end;

procedure TPrepravka.kontrola;
var
    odchylka: real;
    txt: string;
begin
   odchylka := sqr(x) + sqr(y) + sqr(z) - 1.0;
   if odchylka > epsilon then
   begin
     txt := 'Nekonzistentní data po ' + IntToStr(pocet) + ' dodavkach.';
     writeln(txt);
     Halt;
   end;
end;

function TPrepravka.data: string;
begin
  EnterCriticalSection(zamek);
  try
   kontrola;
   Result := '(' + FloatToStr(x) + ', ' + FloatToStr(y) + ', ' + FloatToStr(z) + ')';
  finally
    LeaveCriticalSection(zamek);
  end;
end;

initialization
  prepravka := TPrepravka.Create;
finalization;
  prepravka.Free;
end.

